

timetable = 'https://www.svo.aero/bitrix/timetable/'
terminals = 'https://www.svo.aero/bitrix/map/terminals/'
season_airports = 'https://www.svo.aero/bitrix/season/?locale=en'
season = 'https://www.svo.aero/bitrix/season/list_v2/?city=432&terminal=all&direction=departure&locale=en'
airlines = 'https://www.svo.aero/bitrix/airlines/?perPage=100&page=1&locale=ru'
